import { utils } from './utils.ts'
import { state } from './index.ts'
import type { RPCData } from './types.ts'

export class Client {
    public async listen(data: string) {
        const parsedData = utils.parseData(data)
        await this.transferTo(parsedData)
    }

    private transferTo(data: RPCData) {
        switch (data.to) {
            case utils.environment.CLIENT:
                return this.executeLocal(data)
            case utils.environment.CEF:
            // transfer to cef
        }
    }

    private async executeLocal(data: RPCData) {
        const fnResponse = await state[data.eventName](...data.data)

        const response = {
            data: fnResponse,
            ...data,
        }

        this.sendResponseToServer(response)
    }

    private sendResponseToServer(data: RPCData) {
        const eventName = utils.generateResponseEventName(data.uuid)
        const prepareForTransfer = utils.stringifyData(data)

        mp.events.callRemote(eventName, prepareForTransfer)
    }
}

export const client = new Client()
