import { RPC_LISTENER } from './events.ts'
import { utils } from './utils.ts'

import { client } from './client.ts'
import type { RPCData } from './types.ts'

const environment = utils.getEnvironment()

export const state =
    environment === utils.environment.CEF ? window.rpcEvents : global.rpcEvents

class frameworkRpc {
    constructor() {
        mp.events.add(RPC_LISTENER, async (player: any, data: string) => {
            switch (environment) {
                case utils.environment.UNKNOWN:
                    return

                case utils.environment.CLIENT:
                    player = data
                    return client.listen(player)

                case utils.environment.SERVER:
                case utils.environment.CEF:
            }
        })
    }

    public register<Args extends any[] = unknown[], Return = unknown>(
        eventName: string,
        cb: (...args: Args) => Return,
    ) {
        if (environment === utils.environment.UNKNOWN) return
        state[eventName] = cb
    }

    public callClient<Args extends any[] = unknown[], Return = unknown>(
        player: any,
        eventName: string,
        ...args: Args
    ): Promise<Return | unknown> {
        return new Promise((resolve, reject) => {
            const uuid = utils.generateUUID()

            const data: RPCData = {
                uuid,
                eventName,
                from: environment,
                to: utils.environment.CLIENT,
                data: args,
            }

            player.call(RPC_LISTENER, [utils.stringifyData(data)])
        })
    }
}

const test = new frameworkRpc()
